from azure.storage.blob import BlockBlobService

from .exceptions import NoBlobsFound


class AzureBlobDelete:
    """
    Delete file and folder from Azure blob storage.
    """
    def __init__(self, account_name: str, account_key: str, container_name: str):
        """
        :param account_name:
            Azure storage account name.
        :param account_key:
            Azure storage key.
        :param container_name:
            Azure storage container name, URL will be added automatically.
        """
        self.account_name = account_name
        self.account_key = account_key
        self.container_name = container_name

        self.block_blob_service = BlockBlobService(self.account_name, self.account_key)

    def delete_file(self, file_name: str) -> bool:
        """
        Delete a file from Azure Storage Blob.

        :param file_name:
            Give a file name to delete/
        :rtype: bool
        :returns: ``True`` if a file is deleted.

        >>> from azblobexplorer import AzureBlobDelete
        >>> az = AzureBlobDelete('account name', 'account key', 'container name')
        >>> az.delete_file('file_name.txt')
        True
        """

        self.block_blob_service.delete_blob(self.container_name, file_name)

        return True

    def delete_files(self, file_names: list) -> bool:
        """
        Delete a list of file from Azure Storage Blob.

        :param file_names:
            Give a list of file names to delete/
        :rtype: bool
        :returns: ``True`` if files are deleted.


        >>> from azblobexplorer import AzureBlobDelete
        >>> az = AzureBlobDelete('account name', 'account key', 'container name')
        >>> blob_list = [
        ...     'folder_1/file1.txt',
        ...     'file3.txt'
        ... ]
        >>> az.delete_files(blob_list)
        True
        """

        for file in file_names:
            self.delete_file(file)

        return True

    def delete_folder(self, blob_folder_name: str) -> bool:
        """
        Delete a folder from Azure Storage Blob.

        :param blob_folder_name:
            Give a folder name to delete
        :rtype: bool
        :returns: ``True`` if a folder is deleted.
        :raises NoBlobsFound: If the blob folder is empty or is not found.

        >>> from azblobexplorer import AzureBlobDelete
        >>> az = AzureBlobDelete('account name', 'account key', 'container name')
        >>> az.delete_folder('temp/')
        True
        """

        blobs = list(self.block_blob_service.list_blobs(self.container_name, blob_folder_name))

        if len(blobs) == 0:
            raise NoBlobsFound(
                "There where 0 blobs found with blob path '{}'".format(blob_folder_name))

        for blob in blobs:
            self.delete_file(blob.name)

        return True

    def delete_container(self) -> bool:
        """
        Delete the current container.

        :rtype: bool
        :return: Returns ``True`` is the current container is deleted.

        >>> from azblobexplorer import AzureBlobDelete
        >>> az = AzureBlobDelete('account name', 'account key', 'container name')
        >>> az.delete_container()
        True
        """

        self.block_blob_service.delete_container(self.container_name)

        return True



import os
from pathlib import Path

from azure.storage.blob import BlockBlobService

__all__ = ['AzureBlobUpload']


class AzureBlobUpload:
    """
    Upload a file or a folder.
    """

    def __init__(self, account_name: str, account_key: str, container_name: str):
        """
        :param account_name:
            Azure storage account name.
        :param account_key:
            Azure storage key.
        :param container_name:
            Azure storage container name, URL will be added automatically.
        """
        self.account_name = account_name
        self.account_key = account_key
        self.container_name = container_name

        self.block_blob_service = BlockBlobService(self.account_name, self.account_key)

    def upload_file(self, file_path: str, upload_to: str = None):
        """
        Upload a file to a given blob path.

        :param upload_to:
            Give the path to upload.
        :param file_path:
            Absolute path of the file to upload.

        >>> from azblobexplorer import AzureBlobUpload
        >>> import os
        >>> az = AzureBlobUpload('account name', 'account key', 'container name')
        >>> here = os.path.abspath(os.path.dirname(__file__)) + os.sep
        >>> az.upload_file(os.path.join(here, 'file1.txt'), 'blob_folder/')
        """

        path = Path(file_path)

        if upload_to is None:
            self.block_blob_service.create_blob_from_path(self.container_name, path.name, path)
        else:
            self.block_blob_service.create_blob_from_path(self.container_name,
                                                          upload_to + path.name, path)

    def upload_files(self, files_path: list):
        """
        Upload a list of files.

        :param list files_path:
            A list of files to upload.

        >>> import os
        >>> from azblobexplorer import AzureBlobUpload
        >>> az = AzureBlobUpload('account name', 'account key', 'container name')
        >>> here = os.path.abspath(os.path.dirname(__file__)) + os.sep
        >>> path_list = [
        ...     [os.path.join(here, 'file1.txt'), 'folder_1/'],
        ...     [os.path.join(here, 'file2.txt'), 'folder_2/'],
        ...     os.path.join(here, 'file3.txt')
        ... ]
        >>> az.upload_files(path_list)
        """

        for path in files_path:
            if isinstance(path, list):
                self.upload_file(path[0], path[1])
            else:
                self.upload_file(path)

    def upload_folder(self, folder_path: str, upload_to: str = None):
        """
        Upload a folder to a given blob path.

        :param upload_to:
            Give the path to upload. Default ``None``.
        :param folder_path:
            Absolute path of the folder to upload.

        **Example without "upload_to"**

        >>> import os
        >>> from azblobexplorer import AzureBlobUpload
        >>> here = os.path.abspath(os.path.dirname(__file__)) + os.sep
        >>> az = AzureBlobUpload('account name', 'account key', 'container name')
        >>> az.upload_folder(os.path.join(here, 'folder_name'))

        **Example with "upload_to"**

        >>> import os
        >>> from azblobexplorer import AzureBlobUpload
        >>> here = os.path.abspath(os.path.dirname(__file__)) + os.sep
        >>> az = AzureBlobUpload('account name', 'account key', 'container name')
        >>> az.upload_folder(os.path.join(here, 'folder_name'), upload_to="my/blob/location/")
        """

        path = Path(folder_path)

        if not path.is_dir():
            raise TypeError("The path should be a folder.")

        root_name = path.name

        for _dir, _, files in os.walk(path):
            for file_name in files:
                rel_dir = os.path.relpath(_dir, path)
                rel_folder_path = os.path.join(root_name, rel_dir) + '/'
                abs_path = os.path.join(_dir, file_name)
                if upload_to is None:
                    self.upload_file(abs_path, rel_folder_path)
                else:
                    self.upload_file(abs_path, upload_to + rel_folder_path)




class NoBlobsFound(Exception):
    """
    This exception is raised when there are no blobs.
    """
    def __init__(self, message, errors=None):
        super(NoBlobsFound, self).__init__(message)

        self.errors = errors



import os
from datetime import datetime, timedelta
from pathlib import Path

from azure.storage.blob import BlockBlobService, BlobPermissions

from .exceptions import NoBlobsFound

__all__ = ['AzureBlobDownload']


class AzureBlobDownload:
    """
    Download a file or folder.
    """

    def __init__(self, account_name: str, account_key: str, container_name: str):
        """
        :param account_name:
            Azure storage account name.
        :param account_key:
            Azure storage key.
        :param container_name:
            Azure storage container name, URL will be added automatically.
        """
        self.account_name = account_name
        self.account_key = account_key
        self.container_name = container_name

        self.block_blob_service = BlockBlobService(self.account_name, self.account_key)

    def download_file(self, blob_name: str, download_to: str = None):
        """
        Download a file to a location.

        :param blob_name:
            Give a blob path with file name.
        :param download_to:
            Give a local absolute path to download.
        :raises OSError: If the directory for ``download_to`` does not exists

        >>> from azblobexplorer import AzureBlobDownload
        >>> az = AzureBlobDownload('account name', 'account key', 'container name')
        >>> az.download_file('some/name/file.txt')
        """

        file_dict = self.read_file(blob_name)
        file_name = Path(file_dict['file_name']).name

        if download_to is None:
            write_to = file_name
        else:
            write_to = Path(os.path.join(download_to, file_name))
            write_to.parent.mkdir(parents=True, exist_ok=True)

        with open(write_to, 'wb') as file:
            file.write(file_dict['content'])

    def download_folder(self, blob_folder_name: str, download_to: str = None):
        """
        Download a blob folder.

        :param blob_folder_name:
            Give a folder name.
        :param download_to:
            Give a local path to download.
        :raises NoBlobsFound: If the blob folder is empty or is not found.
        :raises OSError: If the directory for ``download_to`` does not exists

        >>> from azblobexplorer import AzureBlobDownload
        >>> az = AzureBlobDownload('account name', 'account key', 'container name')
        >>> az.download_folder('some/name/file.txt')
        """

        blobs = self.block_blob_service.list_blobs(self.container_name, blob_folder_name)

        if blobs.items == 0:
            raise NoBlobsFound(
                "There where 0 blobs found with blob path '{}'".format(blob_folder_name))

        if download_to is None:
            for blob in blobs:
                name = blob.name
                path = Path(os.path.join(blob_folder_name, name))
                path.parent.mkdir(parents=True, exist_ok=True)
                _blob = self.read_file(name)
                file = open(path, 'wb')
                file.write(_blob['content'])
                file.close()
        else:
            for blob in blobs:
                name = blob.name
                path = Path(os.path.join(download_to, blob_folder_name, name))
                path.parent.mkdir(parents=True, exist_ok=True)
                _blob = self.read_file(name)
                file = open(path, 'wb')
                file.write(_blob['content'])
                file.close()

    def read_file(self, blob_name: str) -> dict:
        """
        Read a file.

        :param blob_name:
            Give a file name.
        :return:
            Returns a dictionary of name, content,

        >>> from azblobexplorer import AzureBlobDownload
        >>> az = AzureBlobDownload('account name', 'account key', 'container name')
        >>> az.read_file('some/name/file.txt')
        {
            'file_name': 'file.txt',
            'content': byte content,
            'file_size_bytes': size in bytes
        }
        """

        blob_obj = self.block_blob_service.get_blob_to_bytes(self.container_name, blob_name)

        return {
            'file_name': blob_obj.name,
            'content': blob_obj.content,
            'file_size_bytes': blob_obj.properties.content_length
        }

    def generate_url(self, blob_name: str, permission: BlobPermissions = BlobPermissions.READ, sas: bool = False) -> str:
        """
        Generate's blob URL. It can also generate Shared Access Signature (SAS) if ``sas=True``.

        :param blob_name: Name of the blob
        :type blob_name: str
        :param permission: Permissions for the data
        :type permission: azure.storage.blob.BlobPermissions
        :param sas: Set ``True`` to generate SAS key
        :type sas: bool
        :return: Blob URL
        :rtype: str
        """

        if sas:
            token = self.block_blob_service.generate_blob_shared_access_signature(
                self.container_name,
                blob_name,
                permission=permission,
                expiry=datetime.utcnow() + timedelta(hours=1)
            )
            return self.block_blob_service.make_blob_url(self.container_name, blob_name, sas_token=token)
        else:
            return self.block_blob_service.make_blob_url(self.container_name, blob_name)

    def generate_url_mime(self, blob_name: str, mime_type: str, sas: bool = False,
                          permission: BlobPermissions = BlobPermissions.READ) -> str:
        """
        Generate's blob URL with MIME type. It can also generate Shared Access Signature (SAS) if ``sas=True``.

        :param blob_name: Name of the blob
        :type blob_name: str
        :param mime_type: MIME type of the application
        :type mime_type: str
        :param sas: Set ``True`` to generate SAS key
        :type sas: bool
        :param permission: Permissions for the data
        :type permission: azure.storage.blob.BlobPermissions
        :return: Blob URL
        :rtype: str
        """

        if sas:
            token = self.block_blob_service.generate_blob_shared_access_signature(
                self.container_name,
                blob_name,
                permission=permission,
                expiry=datetime.utcnow() + timedelta(hours=1),
                content_type=mime_type
            )
            return self.block_blob_service.make_blob_url(self.container_name, blob_name, sas_token=token)
        else:
            return self.block_blob_service.make_blob_url(self.container_name, blob_name)



import codecs
import os

from setuptools import setup, find_packages

from azblobexplorer import __version__

here = os.path.abspath(os.path.dirname(__file__)) + os.sep


def get_requirements(*parts):
    return codecs.open(os.path.join(here, *parts), 'r').read().splitlines()


setup(
    name='azblobexplorer',
    version=__version__,
    install_requires=get_requirements('requirements.txt'),
    packages=find_packages(),
    url='https://github.com/akshaybabloo/azure-blob-explorer-python',
    license='MIT',
    author='Akshay Raj Gollahalli',
    author_email='akshay@gollahalli.com',
    description='Download and upload files to Azure Blob storage.',
    long_description=open('README.md').read(),
    long_description_content_type="text/markdown",
    keywords=['azure', 'secrets'],
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Intended Audience :: Developers',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Utilities'
    ]
)
